
function [globalResults] ...
    = calculatingGlobalResults(folder, NUMBER_OF_IMAGES, NUMBER_OF_TESTS, metaheuristic, segmentationMethods, ...
    allResultsLocalizationCA, allResultsLocalizationDG, allResultsSegmentation, ...
    fileResultsLocalizationDG, fileResultsLocalizationCA,performSegmentation,ImageModality)

global databaseOfMethods;
global metricsUsed;

% The mean, median and std are calculated over all images, i.e. the final
% std is not the mean of the stds per image, it is the std for all images.

numberValues = NUMBER_OF_IMAGES*NUMBER_OF_TESTS;

if ((size(allResultsSegmentation,2)) ~= (numberValues))
    disp('The number of results is not correct!!!');
    globalResults.errorObtained = 1;
else %The number of results is correct
    % Average, median and std of evaluatedRegions in IOECL if applied
    if performSegmentation == 1
        if sum(strcmp(segmentationMethods,'IOECL')) > 0
            temp = zeros(1,numberValues);
            for i=1:numberValues
                temp(i) = allResultsSegmentation{i}.evaluatedRegionsIOECL;
            end;
            globalResults.evaluatedRegions.mean = mean(temp);
            globalResults.evaluatedRegions.median = median(temp);
            globalResults.evaluatedRegions.std = std(temp);
            globalResults.evaluatedRegions.max = max(temp);
            globalResults.evaluatedRegions.min = min(temp);
        end;
        
        % Average, median and std of localizationTime (if it's not applied it will be zero)
        temp = zeros(1,numberValues);
        for i=1:numberValues
            temp(i) = allResultsSegmentation{i}.localizationTime;
        end;
        globalResults.localizationTime.mean = mean(temp);
        globalResults.localizationTime.median = median(temp);
        globalResults.localizationTime.std = std(temp);
        globalResults.localizationTime.max = max(temp);
        globalResults.localizationTime.min = min(temp);
        
        % Now calculate mean, median and std for all segmentation methods
        for j=1:size(databaseOfMethods,2)
            if sum((strcmp(segmentationMethods,databaseOfMethods{j}))) > 0
                for k=1:size(metricsUsed,2)
                    temp1 = zeros(1,numberValues);
                    for i=1:numberValues
                        temp1(i) = allResultsSegmentation{i}.(genvarname(databaseOfMethods{j})).(genvarname(metricsUsed{k}));
                    end;
                    globalResults.(genvarname(databaseOfMethods{j})).(genvarname(metricsUsed{k})).mean = mean(temp1);
                    globalResults.(genvarname(databaseOfMethods{j})).(genvarname(metricsUsed{k})).median = median(temp1);
                    globalResults.(genvarname(databaseOfMethods{j})).(genvarname(metricsUsed{k})).std = std(temp1);
                    globalResults.(genvarname(databaseOfMethods{j})).(genvarname(metricsUsed{k})).max = max(temp1);
                    globalResults.(genvarname(databaseOfMethods{j})).(genvarname(metricsUsed{k})).min = min(temp1);
                end;
            end;
        end;
    else
        globalResults = 0.0;
    end;
    
    % Create file of fitness values, comparing different localization
    % methods
    matrixToPrintDG = [];
    matrixToPrintCA = [];
    label = '';
    if ismember('DE',metaheuristic)
        for i=1:numberValues
            matrixToPrintDG = [matrixToPrintDG allResultsLocalizationDG{i}.fitnessDE];
            matrixToPrintCA = [matrixToPrintCA allResultsLocalizationCA{i}.fitnessDE];
        end;
        label = strcat(label,'DE,');
    end;
    if ismember('PSO',metaheuristic)
        for i=1:numberValues
            matrixToPrintDG = [matrixToPrintDG allResultsLocalizationDG{i}.fitnessPSO];
            matrixToPrintCA = [matrixToPrintCA allResultsLocalizationCA{i}.fitnessPSO];
        end;
        label = strcat(label,'PSO,');
    end;
    if ismember('SS',metaheuristic)
        for i=1:numberValues
            matrixToPrintDG = [matrixToPrintDG allResultsLocalizationDG{i}.fitnessSS];
            matrixToPrintCA = [matrixToPrintCA allResultsLocalizationCA{i}.fitnessSS];
        end;
        label = strcat(label,'SS,');
    end;
    if ismember('SA',metaheuristic)
        for i=1:numberValues
            matrixToPrintDG = [matrixToPrintDG allResultsLocalizationDG{i}.fitnessSA];
            matrixToPrintCA = [matrixToPrintCA allResultsLocalizationCA{i}.fitnessSA];
        end;
        label = strcat(label,'SA,');
    end;
    if ismember('GA',metaheuristic)
        for i=1:numberValues
            matrixToPrintDG = [matrixToPrintDG allResultsLocalizationDG{i}.fitnessGA];
            matrixToPrintCA = [matrixToPrintCA allResultsLocalizationCA{i}.fitnessGA];
        end;
        label = strcat(label,'GA,');
    end;
    
    fileID = fopen(strcat(folder,filesep,fileResultsLocalizationDG,'.txt'),'w');
    label(size(label,2))='';
    fprintf(fileID,'%s\n',label);
    fclose(fileID);
    dlmwrite(strcat(folder,filesep,fileResultsLocalizationDG,'.txt'), matrixToPrintDG', '-append', 'delimiter', ',');
    
    fileID2 = fopen(strcat(folder,filesep,fileResultsLocalizationCA,'.txt'),'w');
    fprintf(fileID2,'%s\n',label);
    fclose(fileID2);
    dlmwrite(strcat(folder,filesep,fileResultsLocalizationCA,'.txt'), matrixToPrintCA', '-append', 'delimiter', ',');
    
end;

% Save values of globalResults, allResultsSegmentation, allResultsLocalizationDG, allResultsLocalizationCA in .mat
save(strcat(folder,'Experiment_',ImageModality,'_',datestr(now, 'yymmdd_HHMMSS'),'.mat'),'globalResults', ...
    'allResultsSegmentation','allResultsLocalizationDG','allResultsLocalizationCA')


