
function [tout3,ORIG_HIPPO_REGION,CROP_HIPPO_REGION,T_MATRIX,fitnesses,siz_ref, P] = ...
    globalRegistration(REFERENCES, show_output, REF_CONT_FOLDER, I1, BIGGEST_REF_IMAGE_SIZE, ...
    type, HIPPO_REGION, MY_OFFSET, REF_FOLDER, res_img)

tic

% Parameter scaling of translateX translateY rotate resizeX resizeY
% (ShearXY, ShearYX can also be included add the end of the vector, but
%   because rotation can be seen as a type of shear transform, the scaling
%       of the rotation or shear transform must be very small)
% lower and upper bounds
lbg=[-50  -50  -0.314  0.75  0.75];
ubg=[50  50  0.314  1.5  1.5];
%parameters of Levenberg-Marquardt for global registration
if show_output <= 0
    display_lm = 'off';
else
    display_lm = 'final';
end
itermax_lm = 2000;

siz_ref = size(REFERENCES);
fitnesses = zeros(2,siz_ref(1)); %this will keep the fitnesses functions (global and local)

orig_width_ref = zeros(siz_ref(1),1);
P = zeros(siz_ref(1),5); %registration values
scale = [1 1 1 1 1];
if show_output > 0
    fprintf(1,'Start Global Registration...\n');
end
ORIG_HIPPO_REGION = zeros(siz_ref(1),4,3);
CROP_HIPPO_REGION = zeros(siz_ref(1),5,3);
T_MATRIX = zeros(siz_ref(1),3,3);

for ref = 1:siz_ref(1)  %for each reference atlas
    actual_reference = sprintf('reference%i',REFERENCES(ref));
    if show_output > 0
        fprintf(1,'%s\n',actual_reference);
    end
    
    reference_filename = sprintf('%s/%s_cont.png',REF_CONT_FOLDER,actual_reference);
    I2=im2double(imread(reference_filename));
    
    if (size(I2,3)>1)
        I2 = rgb2gray(I2);
    end;
    
    orig_width_ref(ref) = size(I2, 2); % we save the original width of the reference we are working with
    if size(I2, 2) < BIGGEST_REF_IMAGE_SIZE(2)
        I2 = fix_image_width(I2, BIGGEST_REF_IMAGE_SIZE(2),0);
        % now the width of this reference will be the same as the biggest one
        % I2 width has been increased adding black columns to both sides of
        % the image, in the middle remains the slice
    end
    doAffine(0,I1, I2, lbg, ubg, type); %initialize by passing the two images, the boundaries and the distance method
    
    % lsqnonlin SOLVE NONLINEAR LEAST-SQUARES (NONLINEAR DATA-FITTING) PROBLEMS
    % - THE FUNCTION WHOSE SUM OF SQUARES IS MINIMIZED:
    % @(x)affine_mask(x,scale,I1,I2,type,lb,ub)
    [xmin tfxmin]=lsqnonlin(@(x)affine_mask(x,scale,I1,I2,type,lbg,ubg),[0.5 0.5 0.5 0.5 0.5],[],[],optimset('Display',display_lm,'TolX',0,'MaxIter',30000,'TolFun',0,'MaxFunEvals', itermax_lm, 'Algorithm','levenberg-marquardt'));
    xmin = xmin.*(ubg-lbg) + lbg;
    %     tfxmin = affine_registration_image(xmin,scale,I1,I2,type);
    
    fitnesses(1,ref) = tfxmin;
    P(ref,:)=xmin;
    % Make the affine transformation matrix.
    % This function make_transformation_matrix.m creates an affine
    % 2D or 3D transformation matrix from translation, rotation, resize and
    % shear parameters
    % inputs (2D)
    %   t: vector [translateX translateY]
    %   r: vector [rotate]   (rotation clockwise about the origin)
    %   s: vector [resizeX resizeY]
    %   h: vector [ShearXY, ShearYX]
    % outputs
    %   M: 2D or 3D affine transformation matrix
    M=make_transformation_matrix(xmin(1:2),xmin(3),xmin(4:5));
    T_MATRIX(ref,:,:) = M;
    Minv=make_transformation_matrix([xmin(2) xmin(1)],-xmin(3),[xmin(5) xmin(4)]); %this strange behaviour is due to matlab's matrix representation
    
    % Function affine_transform, is a wrapper of the (mex) functions
    % affine_transform_2d_double and affine_transform_3d mex functions
    %   mode: If 0: linear interpolation and outside pixels set to nearest pixel
    %            1: linear interpolation and outside pixels set to zero
    %            2: cubic interpolation and outside pixels set to nearest pixel
    %            3: cubic interpolation and outside pixels set to zero
    Icor=affine_transform(I1,M,3); % 3 stands for cubic interpolation
    
    ref_hippo = HIPPO_REGION.(actual_reference); %load ROI coordinates
    
    %add offset to enlarge the roi
    ref_hippo = ref_hippo + MY_OFFSET;
    ref_hippo = [ref_hippo(1) ref_hippo(2) 1; ref_hippo(1) ref_hippo(4) 1; ref_hippo(3) ref_hippo(2) 1; ref_hippo(3) ref_hippo(4) 1]; %from coordinates to points
    % Representation of the points (being (0,0) the upper left corner)
    % (xmin, ymin)--------------(xmax,ymin)
    %       |                          |
    %       |                          |
    %       |                          |
    %       |                          |
    % (xmin, ymax)--------------(xmax,ymax)
    ref_hippo(:,1) = ref_hippo(:,1) + (BIGGEST_REF_IMAGE_SIZE(2) - orig_width_ref(ref)) / 2.0;  %move if you added bands on side
    ref_to_draw = [ref_hippo(1,:);ref_hippo(2,:);ref_hippo(4,:);ref_hippo(3,:);ref_hippo(1,:)]; %to draw a rectangle on images
    change_coords = repmat([BIGGEST_REF_IMAGE_SIZE(2)./2 , BIGGEST_REF_IMAGE_SIZE(1)./2 , 0],4,1);
    ref_hippo = ref_hippo - change_coords;
    orig_hippo = (Minv*ref_hippo')'; %back transformation
    orig_hippo = orig_hippo + change_coords;
    orig_to_draw = [orig_hippo(1,:);orig_hippo(2,:);orig_hippo(4,:);orig_hippo(3,:);orig_hippo(1,:)];
    ORIG_HIPPO_REGION(ref,:,:) = orig_hippo;  %remember original and trasformed roi for the next cycle
    CROP_HIPPO_REGION(ref,:,:) = ref_to_draw;
    if show_output > 2 %show some images if desired
        entire_ref_filename = sprintf('%s/%s.png',REF_FOLDER,actual_reference);
        I1show = res_img; %load grayscale images and adjust the width
        I2show = im2double(rgb2gray(imread(entire_ref_filename)));
        I1show = fix_image_width(I1show, BIGGEST_REF_IMAGE_SIZE(2),1);
        I2show = fix_image_width(I2show, BIGGEST_REF_IMAGE_SIZE(2),1);
        Icorshow=affine_transform(I1show,M,3);
        figure_title = sprintf('Contour Not Registered, %d | Green: reference/rigid image | Red: target/deformable image | Yellow: intersection reference and target',REFERENCES(ref));
        I2_color = repmat(uint8(255.*I2),[1 1 3]);
        I2_color(:,:,1) = 0;
        I2_color(:,:,3) = 0;
        figure('Name',figure_title,'NumberTitle','off'); imshow(I2_color)
        I1_color = repmat(uint8(255.*I1),[1 1 3]);
        I1_color(:,:,3) = 0;
        I1_color(:,:,2) = 0;
        hold on
        h = imshow(I1_color);
        set(h, 'AlphaData', 0.5)
        
        figure_title = sprintf('Image Not Registered, %d',REFERENCES(ref));
        figure('Name',figure_title,'NumberTitle','off'); imshow(I2show)
        hold on
        line(orig_to_draw(:,1),orig_to_draw(:,2));
        h = imshow(I1show);
        set(h, 'AlphaData', 0.5)
        
        figure_title = sprintf('Contour Registered, %d | Green: reference/rigid image | Red: target/registered image | Yellow: intersection reference and target',REFERENCES(ref));
        figure('Name',figure_title,'NumberTitle','off'); imshow(I2_color)
        hold on
        Icor_color = repmat(uint8(255.*Icor),[1 1 3]);
        Icor_color(:,:,3) = 0;
        Icor_color(:,:,2) = 0;
        h = imshow(Icor_color);
        set(h, 'AlphaData', 0.5)
        
        figure_title = sprintf('Image Registered, %d',REFERENCES(ref));
        figure('Name',figure_title,'NumberTitle','off'); imshow(I2show)
        hold on
        line(ref_to_draw(:,1),ref_to_draw(:,2));
        h = imshow(Icorshow);
        set(h, 'AlphaData', 0.5)
    end
    
end %for global registration
tout3 = toc;%time: global registration