
function [tout4, sum_variance, calculate_local, fitnesses, I1show] = ...
    localRegistration(fitnesses,CROP_HIPPO_REGION, T_MATRIX, REFERENCES, WORST_GLOBAL_FITNESS_FACTOR, ...
    REF_FOLDER, type, show_output, siz_ref, res_img, BIGGEST_REF_IMAGE_SIZE, f, P,out_dir_name, image_name)

tic
%pso parameters for local registration
psoOptions2 = get_psoOptions;

psoOptions2.Flags.ShowViz = 0;
psoOptions2.Flags.Neighbor = 0;

psoOptions2.Vars.SwarmSize = 40;
psoOptions2.Vars.Dim = 5;
psoOptions2.Vars.Iterations = 45;
psoOptions2.Vars.ErrGoal = 0.001;

psoOptions2.SParams.c1 = 2.05;
psoOptions2.SParams.c2 = 2.05;
psoOptions2.SParams.w_start = 1.0;
psoOptions2.SParams.w_end = 0.1;
% psoOptions2.SParams.w_varyfor = 1;
psoOptions2.SParams.Vmax = 1;

psoOptions2.Obj.f2eval = 'doAffine';
psoOptions2.Obj.lb = 0;
psoOptions2.Obj.ub = 1;
% These two variables are used only during the local search
lbl = [-14  -14  -0.314/2  0.9  0.9];% boundaries
ubl = [14  14  0.314/2  1.1  1.1];

psoOptions2.Terminate.Iters = 1;
psoOptions2.Terminate.Err = 0;

psoOptions2.Save.Interval = 0;
if show_output > 0
    psoOptions2.Disp.Interval = 10; %display results frequency
else
    psoOptions2.Disp.Interval = 0;
end
psoOptions2.Disp.Footer = 1;
psoOptions2.Disp.Progress = 1;

sum_variance = 0.0; %variance of selected rois
PERC_MIN_SAT = 0.7; %saturate background
PERC_MAX_SAT = 0.95; %saturate possible parts darker than hippocampus

%don't do the local registration on images with a very low global fitness
calculate_local = fitnesses(1,:)<(min(fitnesses(1,:))*WORST_GLOBAL_FITNESS_FACTOR);
% fitnesses contains, in the first row, the results of global
% registration and, in the second row, the results of local
% registration

if show_output > 0
    fprintf(1,'Start Local Registration...\n');
end
for ref = 1:siz_ref(1)  %for all reference atlases
    if calculate_local(ref) %only for the ones with good global fitness
        actual_reference = sprintf('reference%i',REFERENCES(ref));
        if show_output > 0
            fprintf(1,'%s\n',actual_reference);
        end
        entire_ref_filename = sprintf('%s/%s.png',REF_FOLDER,actual_reference);
        ref_to_draw = reshape(CROP_HIPPO_REGION(ref,:,:),5,3);
        minx = min(ref_to_draw(:,1));
        miny = min(ref_to_draw(:,2));
        width = max(ref_to_draw(:,1))-minx;
        height = max(ref_to_draw(:,2))-miny;
        
        %load images and crop hippocampi
        I1show = res_img;%im2double(rgb2gray(imread(entire_image_filename)));
        I2show = im2double(rgb2gray(imread(entire_ref_filename)));
        I1show = fix_image_width(I1show, BIGGEST_REF_IMAGE_SIZE(2),1);
        I2show = fix_image_width(I2show, BIGGEST_REF_IMAGE_SIZE(2),1);
        M = reshape(T_MATRIX(ref,:,:),3,3);
        Icorshow=affine_transform(I1show,M,1);
        hip1 = imcrop(Icorshow,[minx miny width height]);
        hip2 = imcrop(I2show,[minx miny width height]);
        %
        %         size(hip1)
        %         figure,imshow(hip1)
        %         size(hip2)
        %         figure,imshow(hip2)
        
        sum_variance = sum_variance + std(hip1(:));
        %here we can set a normalization-equalization of images
        
        hip1 = 1-hip1;hip2=1-hip2;
        %             hip1_filename = sprintf('%s/%s_localreg1pre_ref%s.png',out_dir_name,image_name,num2str(ref));
        %             imwrite(hip1, hip1_filename,'png');
        %             hip2_filename = sprintf('%s/%s_localreg2pre%s.png',out_dir_name,image_name,num2str(ref));
        %             imwrite(hip2, hip2_filename,'png');
        imrow = sort(hip1(:));
        bottom = imrow(ceil((PERC_MIN_SAT)*length(imrow)));
        top = imrow(ceil((PERC_MAX_SAT)*length(imrow)));
        hip1 = (hip1-bottom)/(top-bottom);
        hip1(hip1<0) = 0;
        hip1(hip1>1) = 1;
        imrow = sort(hip2(:));
        bottom = imrow(ceil((PERC_MIN_SAT)*length(imrow)));
        top = imrow(ceil((PERC_MAX_SAT)*length(imrow)));
        hip2 = (hip2-bottom)/(top-bottom);
        hip2(hip2<0) = 0;
        hip2(hip2>1) = 1;
        %             hip1_filename = sprintf('%s/%s_localreg1post%s.png',out_dir_name,image_name,num2str(ref));
        %             imwrite(hip1, hip1_filename,'png');
        %             hip2_filename = sprintf('%s/%s_localreg2post%s.png',out_dir_name,image_name,num2str(ref));
        %             imwrite(hip2, hip2_filename,'png');
        doAffine(0,hip1, hip2, lbl, ubl, type); %initialize by passing the two images and the distance method
        [tfxminh, xminh] = pso_proposed(psoOptions2); %NOTE: in affine_registration_image.m line 42 you can change the method for interpolation and filling
        xminh = xminh.*(ubl-lbl) + lbl;

        LOCAL_TRANSFORM.(actual_reference) = xminh;

        fitnesses(2,ref) = tfxminh;
        
        if show_output > 2
            Mh=make_transformation_matrix(xminh(1:2),xminh(3),xminh(4:5));
            hipcor = affine_transform(hip1, Mh, 1);
            out_hippoc_name = sprintf('%s/%s_hippo_%dRegistered.png',out_dir_name, image_name,REFERENCES(ref));
            figure,imshow(hipcor*0.5 + hip2*0.5);
            imwrite(hipcor*0.5 + hip2*0.5, out_hippoc_name,'png');
            out_hippo_name = sprintf('%s/%s_hippo_%d.png',out_dir_name, image_name,REFERENCES(ref));
            figure,imshow(hip1*0.5 + hip2*0.5);
            imwrite(hip1*0.5 + hip2*0.5, out_hippo_name,'png');                    
        end
        
    else
        fitnesses(2,ref) = 100;
        xminh = [0 0 0 0 0];
    end
    if show_output > -1
        fprintf(f,'%d\t%f\t%f\t[%f %f,%f,%f %f]\t[%f %f,%f,%f %f]\n',REFERENCES(ref), fitnesses(1,ref), fitnesses(2,ref),P(ref,1),P(ref,2),P(ref,3),P(ref,4),P(ref,5),xminh(1),xminh(2),xminh(3),xminh(4),xminh(5));
    end
end %local registration
tout4 = toc; % time: local registration