
function N = Normals2D(Points)
% This function calculates the normals, of the line points
% using the neighbouring points of each contour point, and 
% forward an backward differences on the end points. 
%
% N=LineNormals2D(V,L)
%
% inputs,
%   V : List of points 2 x M
%
% outputs,
%   N : The normals of the Points 2 x M
%

	T = [ Points(:,1) - Points(:,end), Points(:, 2:end) - Points(:,1:end-1) ];

    P = zeros(size(T));
    P(1,:) = -T(2,:);
    P(2,:) = T(1,:);
    
    N = [ P(:, end) + P(:, 1) + P(:, 2) + P(:, 3), ...
          P(:, 1:end-3) + P(:, 2:end-2) + P(:, 3:end-1) + P(:, 4:end), ...
          P(:, end-2) + P(:, end-1) + P(:, end) + P(:, 1), ...
          P(:, end-1) + P(:, end) + P(:, 1) + P(:, 2) ];
    
    L = sqrt(N(1,:).^2 + N(2,:).^2);
    N(1,:) = N(1,:)./L;
    N(2,:) = N(2,:)./L;
    
end