%==========================================================================
%
%   Hybrid image segementation method
%
%--------------------------------------------------------------------------
%   Usage of varibles:
%   input:
%       Img         = any gray/double/RGB input image
%       prior       = segmentation prior
%       num_iter    = total number of iterations
%       wCV         = weight of the C&V term
%       wPrior      = weight of the prior term
%       wEdge       = weight of the edge term
%       wLength          = weight of length term
%
%   output:
%       phi        = updated level set function
%
%--------------------------------------------------------------------------
%
% Description:
%
%==========================================================================

function seg = hybrid_method_withVFC(Img, prior, num_iter, wCV, wPrior, wEdge, wLength)
%%
%-- Default settings
%   length term wLength = 0.2
if(~exist('wLength','var'))
    wLength=0.2;
end

if(wEdge~=0) %if wEdge is different to zero
    addpath(fullfile('segmentationMethods','AMT','AMT'));
    % Edge Force
    % For the VFC it's necessary an edge map, not a gradient map
    Img2 = edge(Img,'canny');
    % In this case, we're using a simple edge detector (Canny), and the
    % idea is to have the background in black and the
    % boundary in white. MAYBE WE'LL NEED TO SMOOTH THE IMAGE.
    K = AM_VFK(2,  max(size(Img)), 'power',1.8);
    Fext = AM_VFC(Img2, K, 1);
    %     figure(1)
    %     imshow(Img);
    %     colormap gray;
    %     hold on
    %     H = quiver(Fext(:,:,1), Fext(:,:,2), .5);
    %     hold off
    %     title('Vector Field Convolution over image')
else %if wEdge is equal to zero -> we don't even need to calculate VFC
    force_edge = zeros(size(Img));
end

%-- End default settings

mask = maskcircle2(Img,'large');
% mask = mask(:,:,1);
% mask(500:600,500:600)=1;

if size(Img,3)== 3
    P = rgb2gray(Img);
    P = double(P);
else
    P = double(Img);
end

%-- End Initializations on input image Img and mask

%%
%--   Core function
%-- SDF
%   Get the distance map of the initial mask
mask = mask(:,:,1);
mask(mask>=0.5) = 1;
mask(mask<=0.5) = 0;
phi0 = double_bwdist(mask) -double_bwdist(1-mask) + im2double(mask) -.5;

%   initial force, set to eps to avoid division by zeros
force = eps;

% %-- Display settings
% figure(1);
% subplot(3,3,1); imshow(Img); title('Input Image');
% subplot(3,3,2); contour(flipud(phi0), [0 0], 'r','LineWidth',1); title('initial contour');
% subplot(3,3,3); title('Segmentation');
% %-- End Display original image and mask

%TO BE REMOVED
L = P;
PX = im2double(prior);


%-- Main loop
for n=1:num_iter
    
    
    % Plain Chan & Vese
    force_image = 0; %
    inidx = find(phi0>=0);
    outidx = find(phi0<0);
    c1 = sum(sum(L.*Heaviside(phi0)))/(length(inidx)+eps); % average inside of Phi0
    c2 = sum(sum(L.*(1-Heaviside(phi0))))/(length(outidx)+eps); % verage outside of Phi0
    force_image = -(L-c1).^2+(L-c2).^2 + force_image;
    
    % Prior force
    % Computed as C&V on the prior image; currently without any
    % optimizations
    force_prior = 0;
    c1 = sum(sum(PX.*Heaviside(phi0)))/(length(inidx)+eps); % average inside of Phi0
    c2 = sum(sum(PX.*(1-Heaviside(phi0))))/(length(outidx)+eps); % verage outside of Phi0
    force_prior = -(PX-c1).^2+(PX-c2).^2 + force_prior;
    
    % calculate the total force of the image
    force = wLength*kappa(phi0)./max(max(abs(kappa(phi0)))) + wCV*force_image + wPrior*force_prior;
    
    % normalized the force
    force = force./(max(max(abs(force))));
    
    
    %     figure(1);
    %     subplot(3,3,4); imshow(force_image); title('CV term');
    %     subplot(3,3,5); imshow(force_prior); title('Prior term');
    %     subplot(3,3,7); imshow(force); title('Total force');
    %
    % get stepsize dt
    dt = 5;
    
    % get parameters for checking whether to stop
    old = phi0;
    %     ACORDARME DE DESCOMENTAR ESTO!!!phi0 = phi0 + dt.*force;
    
    % Compute the EDGE component of the force
    
    %     figure,imshow(phi0)
    
    [zeroLS Contour] = extractZeroLS(phi0);
    %     figure,imshow(zeroLS);
    % IMPORTANT: removes duplicates, don't change the order of the elements
    data = round(Contour)';
    [junk,idx] = unique(data,'rows');
    Contour = data(sort(idx),:)';
    % %     Normals = Normals2D(Contour);
    % %
    % %     force_list = zeros(2,length(Contour));
    % %     for i = 1:length(Contour)
    % %         point = Contour(:,i);
    % %         dotNF = dot( Normals(:,i), Fext(point));
    % %         if dotNF > 0
    % %             force_list(:,i) = Normals(:,i) * dotNF;
    % %         else
    % %             force_list(:, i) = -Normals(:,i) * dotNF;
    % %         end;
    % %     end;
    % %     %     figure(3);
    % %     %
    % %     %     imshow(zeroLS);
    % %     %     hold on
    % %     %     quiver(Contour(1,:), Contour(2,:),Normals(1,:), Normals(2,:));
    % %
    % %     figure(3);
    % %     imshow(zeroLS);
    % %     hold on
    % %     quiver(Contour(1,:), Contour(2,:),force_list(1,:), force_list(2,:));
    % %     %    return;
    % %
    % %     % Apply the edge force, create a new phi
    % %     new_phi = -1 * ones(size(phi0));
    % %
    % %     %TEST
    % %
    % %     force_list = force_list * 10;
    % %
    % %
    % %     for i = 1:length(Contour)
    % %         new_phi( round(Contour(2,i) + force_list(2,i) ), round(Contour(1,i) + force_list(1,i))) = 0;
    % %     end;
    % %     figure(); imshow(zeroLS, []);
    % %     figure(); imshow(new_phi,[]);
    
    SNAKE_ITER = 5;
    tau = 0.5;
    beta = 0;
    alpha = 0.5;
    paso=1;%size(Contour,2)/round(size(Contour,2)*0.4);%0.4
    vert = Contour(:,1:paso:end)';
    %     figure,imshow(Img)
    %     h = AC_display(vert,'close','r-');
    %     set(h,'LineWidth',1)
    
    vert2 = AC_deform(vert,alpha,beta,tau,Fext,SNAKE_ITER);
    vert3 = AC_remesh(vert2,.5);
    
    figure(1),imshow(Img)
    %     h = AC_display(vert3,'close','r-');
    vertex = vert3([1:end,1],:);
    hold on
    h=plot(vertex(:,1),vertex(:,2),'r-');
    set(h,'LineWidth',1)
    hold off
    
    % Now, re-create the zeroLS
    newContour = round(vert3)';
    data = round(newContour)';
    [junk,idx] = unique(data,'rows');
    newContour = data(sort(idx),:)';
    zeroLS2 = zeros(size(phi0));
    zeroLS2(sub2ind(size(phi0), round(newContour(2,:)),round(newContour(1,:)))) = 1;
    
    % Reinit signed distance function
    %ALWAYS REINIT
    %Fill zeroLS2 lo have something like mask
    maskzeroLS2 = imfill(zeroLS2,'holes');
    maskzeroLS2(maskzeroLS2>=0.5) = 1;
    maskzeroLS2(maskzeroLS2<=0.5) = 0;
    phi0 = double_bwdist(maskzeroLS2) -double_bwdist(1-maskzeroLS2) + im2double(maskzeroLS2) -.5;
    
    new = phi0;
    %    indicator = checkstop(old, new, dt);
    
    % intermediate output
%     if(mod(n,3) == 0)
%         showphi(Img, phi0, n);
%     end;
end;
% showphi(Img,phi0,n);

seg = phi0 > 0;

% subplot(3,3,9); imshow(seg); title('Global Region-Based Segmentation');
end
