function [tout13, newmask, individualResults] = ...
    expansionWithClassifiers(SaveSegmentationEvolution, maskpoly, newmask, orig_im, ...
    out_dir_name,image_name, REFERENCES,best_ref,timestamp, im2, individualResults)

tic
% CODES FOR REFINEMENT
%now, we have the segmentation over the windows created by the model
%and we want to extend it. It is a while, repeated until there
%are no intersections between the mask and the windows.
WIN_RADIUS = 150; %radius of the circle-shaped window
MIN_SIZE_CONT = 8; %if the segment is too short, don't use it
addedWindows = true; %stopping criterion: when no windows are added to the mask, stop
n_wide=0; %this was used in the previous version because sometimes there were loops, but now it shouldn't be
borderExpanded = zeros(size(orig_im)); %remember which borders you have already expanded in order to avoid loops

while addedWindows
    addedWindows = 0;
    n_wide = n_wide + 1;
    %             fprintf(1,'%d ',n_wide);
    edgemask = boundaryBW(maskpoly>0);
    edgesegm = boundaryBW(newmask);
    expandme = edgesegm & edgemask; %expandme contains the lines where the localization has been stopped by the size of the window
    cc2 = bwconncomp(expandme);
    numPixels2 = cellfun(@numel,cc2.PixelIdxList);
    %             border = zeros(size(orig_im));
    for i=1:length(numPixels2) %analizza i possibili punti da espandere
        if numPixels2(i)<=MIN_SIZE_CONT %remove isolated points or very small lines
            expandme(cc2.PixelIdxList{i}) = 0;
        else
            borderExpanded(cc2.PixelIdxList{i})=1;
            addedWindows=1; %flag activated
            center = midpoint_segment(cc2.PixelIdxList{i}, size(orig_im)); %find the center of the segment, around whom the window will be built
            minx = max([floor(center(2))-WIN_RADIUS,1]);
            maxx = min([floor(center(2))+WIN_RADIUS,size(orig_im,1)]);
            miny = max([floor(center(1))-WIN_RADIUS,1]);
            maxy = min([floor(center(1))+WIN_RADIUS,size(orig_im,2)]);
            xys2= circle2(center,WIN_RADIUS,10); %create circular window
            tmpMask = poly2mask(xys2(1,:),xys2(2,:),size(orig_im,1),size(orig_im,2)); %create the mask
            tmp_im = im2.*(tmpMask);
            tmp_im(tmp_im==0) = 2;
            tmp_im = 1 - tmp_im;
            level = graythresh(tmp_im(tmp_im>-1)); %binarize
            tmp_BW = im2bw(tmp_im,level);
            CC = bwconncomp(tmp_BW,8);
            numPixels = cellfun(@numel,CC.PixelIdxList);
            [biggest,idx] = max(numPixels); %select the biggest foreground element
            for ii=1:length(numPixels)
                if ii~=idx
                    tmp_BW(CC.PixelIdxList{ii}) = 0; %remove smaller connected components
                end
            end
            newmask(newmask>1)=1;
            check_useless=sum(sum((1-newmask).*tmp_BW)); %sometimes nothing is added to the mask, save time
            
            if ~check_useless
                fprintf(1,'Nothing added!\n');
                individualResults.evaluatedRegionsIOECL = individualResults.evaluatedRegionsIOECL + 1;
                %                     figure, imshow(tmp_BW + newmask*0.1 + orig_im);
                %                     waitforbuttonpress
            else
                if ~classifyWindowTexture(orig_im(minx:maxx,miny:maxy),tmp_BW(minx:maxx,miny:maxy)) %check if the foreground is hippocampus
                    fprintf(1,'Discarded Part.\n');
                    %                     figure, imshow(tmp_BW + tmpMask*0.05 + orig_im);
                    %                     waitforbuttonpress;
                    tmp_BW = zeros(size(orig_im));
                    individualResults.evaluatedRegionsIOECL = individualResults.evaluatedRegionsIOECL + 1;
                    %when the part is discarded, we should try with half size of the window
                else
                    fprintf(1,'Added Part.\n');
                    individualResults.evaluatedRegionsIOECL = individualResults.evaluatedRegionsIOECL + 1;
                end
            end
            %                     figure
            %                     imshow(0.1*tmpMask + orig_im);
            %                     waitforbuttonpress %click!
            %                     close(gcf)
            newmask = newmask + tmp_BW;
            newmask(newmask>1)=1;
            
            if SaveSegmentationEvolution==1
                % CODES TO SAVE THE IMAGES WITH THE EVOLUTION OF THE SEGMENTATION
                RGBnewmask = repmat(uint8(255.*newmask),[1 1 3]);
                RGBnewmask(:,:,3) = 0;
                RGBnewmask(:,:,2) = 0;
                RGBorig_im = repmat(uint8(255.*orig_im),[1 1 3]);
                index_find=find(RGBnewmask(:,:,1)==255 & RGBnewmask(:,:,2)==0 & RGBnewmask(:,:,3)==0);
                RGBorig_im(index_find) = 255;
                RGBorig_im(index_find+size(RGBorig_im,1)*size(RGBorig_im,2)) = 0;
                RGBorig_im(index_find+2*size(RGBorig_im,1)*size(RGBorig_im,2)) = 0;
                nameFileEvolutione = sprintf('%s/%s_IterOtsu_Evolution%d_ECL.png',out_dir_name,image_name, savingNumberImage);
                imwrite(RGBorig_im, nameFileEvolutione,'png');
                savingNumberImage = savingNumberImage + 1 ;
            end;
            
            
            
            maskpoly = maskpoly + tmpMask;
            
        end
    end
    if individualResults.evaluatedRegionsIOECL > 25 % for avoiding infinite loops
        addedWindows=0;
    end;
end %endwhile
tout13 = toc;


hippo_filename = sprintf('%s/%s_newmask_IterOtsuRefined_%d_%s.png',out_dir_name,image_name, REFERENCES(best_ref),timestamp); %save extracted ROI

imwrite(newmask, hippo_filename,'png');

hippo_filename = sprintf('%s/%s_orig_im_newmask_IterOtsuRefined_%d_%s.png',out_dir_name,image_name, REFERENCES(best_ref),timestamp); %save extracted ROI

imwrite(orig_im.*newmask, hippo_filename,'png');
save(strcat(out_dir_name,'/',image_name,'_IterOtsuRefined.mat'),'newmask');
% in borderExpanded & boundaryBW(newmask) you can find the
% non-expanded borders (blocked by classifyWindowTexture: we could
% try with smaller windows (WIN_RADIUS/2)


%         figure,
%         ax(1)=subplot(1,3,1); imshow(orig_im.*oldmask);
%         ax(2)=subplot(1,3,2); imshow(orig_im.*(newmask-oldmask));
%         ax(3)=subplot(1,3,3); imshow(orig_im.*newmask + (borderExpanded & boundaryBW(newmask)));
%         linkaxes(ax,'xy');
