function [tout20, newmask] = ...
    segmentationDS(orig_im, out_dir_name,image_name, REFERENCES,best_ref,timestamp, phase1,ImageModality)

%% DEFORMABLE SEGMENTATION (SEGMENTATION THROUGH AFFINE REGISTRATION USING
%% SCATTER SEARCH AS OPTIMIZATION METHOD, AND AFTER USING B-SPLINES): NON-DETERMINISTIC METHOD

%% IMPORTANT NOTES:
% CONVERT FILES (reference, orig_im, and mask) TO THE PROPER FORMAT (mhd) WITH ConvertImages.exe image_file.
% THESE IMAGES SHOULD BE IN A DIFFERENT FOLDER THAN THE COMMONLY USED,
% SINCE THEY ARE USED ONLY WITH ELASTIX.

ELASTIX_GROUND_TRUTH = 'elastixFolders/ground-truth';
ELASTIX_MASKS = 'elastixFolders/masks';
ELASTIX_ROI_PROCESSED = 'elastixFolders/ROI_processed';
source_image_nameGT = sprintf('%s/%s.mhd',ELASTIX_GROUND_TRUTH,image_name);
if strcmp(ImageModality,'ISH')
    source_image_nameMask = sprintf('%s/reference%d_mask.mhd',ELASTIX_MASKS,REFERENCES(best_ref));
end;
source_image_nameROIprocessed = sprintf('%s/%s.mhd',ELASTIX_ROI_PROCESSED,image_name);

%% Method
tic
if strcmp(ImageModality,'ISH')
    flag = (sum((exist(source_image_nameGT, 'file') + exist(source_image_nameMask, 'file') + exist(source_image_nameROIprocessed, 'file'))) == 6);
else
    flag = (sum((exist(source_image_nameGT, 'file') + exist(source_image_nameROIprocessed, 'file'))) == 4);
    databaseReferencesElastix_CT_MRI
    [truefalse, index] = ismember(image_name, ImagesTable);
    referenceToUseName = ReferencesTable(index);
end;
if flag
    out_dir_name1 = sprintf('%s/%s',out_dir_name, 'outputAffine');
    out_dir_name3 = sprintf('%s/%s',out_dir_name, 'outputTransformBSpline');
    mkdir(out_dir_name3);
    out_dir_name4 = sprintf('%s/%s',out_dir_name, 'outputBSpline');
    mkdir(out_dir_name4);
    
    % if we already have the outputAffine folder we can assume that AS
    % was aready executed. If not, then we enter in this IF
    if (exist(out_dir_name1,'dir') ~= 7)
        mkdir(out_dir_name1);
        tic
        
        if strcmp(ImageModality,'ISH')
            % #First, perform the affine registration (with mask)
            %         elastix -m reference_image.mhd -f target_image.mhd -mMask
            %         mask_reference_image.mhd -out output_directory -p parametersAffine.txt
            command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'.mhd'), ...
                ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
                ' -mMask ', strcat(strrep(ELASTIX_MASKS, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'_mask.mhd'), ...
                ' -out ', strrep(out_dir_name1, '/', '\') , ' -p elastixFolders\parametersAffineISH.txt'];
        elseif strcmp(ImageModality,'Lungs')
            %             if image_name(1) == 'l'
            % elastix -m reference_image -f target_image -out
            % output_directory_affine -p parametersAffine.txt
            command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\',char(referenceToUseName),'.mhd'), ...
                ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
                ' -out ', strrep(out_dir_name1, '/', '\') , ' -p elastixFolders\parametersAffineLungs.txt'];
        elseif strcmp(ImageModality,'Knee')
            % elastix -m reference_image -f target_image -out
            % output_directory_affine -p parametersAffine.txt
            command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\',char(referenceToUseName),'.mhd'), ...
                ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
                ' -out ', strrep(out_dir_name1, '/', '\') , ' -p elastixFolders\parametersAffineKnee.txt'];
            %             end;
        elseif strcmp(ImageModality,'MRI')
            % elastix -m reference_image -f target_image -out
            % output_directory_affine -p parametersAffine.txt
            command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\',char(referenceToUseName),'.mhd'), ...
                ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
                ' -out ', strrep(out_dir_name1, '/', '\') , ' -p elastixFolders\parametersAffineMRI.txt'];
        end;
        system(command); %execute external command
        phase1 = toc;
    end;
    
    tic
    if strcmp(ImageModality,'ISH')
        % #Then, perform the bspline registration (without mask, and note the different output directory)
        % elastix -m reference_image.mhd -f target_image.mhd -out output_directory_bsplines -p
        % parametersBSplines.txt -t0 output_directory_affine/TransformParameters.0.txt
        command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'.mhd'), ...
            ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
            ' -out ', strrep(out_dir_name4, '/', '\'),' -p elastixFolders\parametersBSplinesISH.txt', ...
            ' -t0 ', strcat(strrep(out_dir_name1, '/', '\'),'\TransformParameters.0.txt')];
    elseif strcmp(ImageModality,'Lungs')
        %         if image_name(1) == 'l'
        % elastix -m reference_image -f target_image -out
        % output_directory_bsplines -p parametersBSplinesLungs.txt -t0
        % output_directory_affine/TransformParameters.0.txt
        command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\',char(referenceToUseName),'.mhd'), ...
            ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
            ' -out ', strrep(out_dir_name4, '/', '\'),' -p elastixFolders\parametersBSplinesLungs.txt', ...
            ' -t0 ', strcat(strrep(out_dir_name1, '/', '\'),'\TransformParameters.0.txt')];
    elseif strcmp(ImageModality,'Knee')
        % elastix -m reference_image -f target_image -out
        % output_directory_bsplines -p parametersBSplinesKnee.txt -t0
        % output_directory_affine/TransformParameters.0.txt
        command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\',char(referenceToUseName),'.mhd'), ...
            ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
            ' -out ', strrep(out_dir_name4, '/', '\'),' -p elastixFolders\parametersBSplinesKnee.txt', ...
            ' -t0 ', strcat(strrep(out_dir_name1, '/', '\'),'\TransformParameters.0.txt')];
        %         end;
    elseif strcmp(ImageModality,'MRI')
        % elastix -m reference_image -f target_image -out
        % output_directory_bsplines -p parametersBSplinesMRI.txt -t0
        % output_directory_affine/TransformParameters.0.txt
        command = ['elastixFiles\elastix.exe -m ', strcat(strrep(ELASTIX_ROI_PROCESSED, '/', '\'),'\',char(referenceToUseName),'.mhd'), ...
            ' -f ', strrep(source_image_nameROIprocessed, '/', '\'), ...
            ' -out ', strrep(out_dir_name4, '/', '\'),' -p elastixFolders\parametersBSplinesMRI.txt', ...
            ' -t0 ', strcat(strrep(out_dir_name1, '/', '\'),'\TransformParameters.0.txt')];
    end;
    system(command); %execute external command
    
    if strcmp(ImageModality,'ISH')
        % # Apply the resulting transformation to the ground truth
        %     transformix -in ground_truth_reference_image.mhd -out
        %     other_output_directory -tp output_directory_bsplines/TransformParameters.0.txt
        command = ['elastixFiles\transformix.exe -in ', strcat(strrep(ELASTIX_GROUND_TRUTH, '/', '\'),'\reference',num2str(REFERENCES(best_ref)),'.mhd'), ...
            ' -out ', strrep(out_dir_name3, '/', '\'), ...
            ' -tp ', strcat(strrep(out_dir_name4, '/', '\'),'\TransformParameters.0.txt')];
    else
        % transformix -in ground_truth_reference_image -out
        % other_output_directory -tp
        % output_directory_bsplines/TransformParameters.0.txt
        command = ['elastixFiles\transformix.exe -in ', strcat(strrep(ELASTIX_GROUND_TRUTH, '/', '\'),'\',char(referenceToUseName),'.mhd'), ...
            ' -out ', strrep(out_dir_name3, '/', '\'), ...
            ' -tp ', strcat(strrep(out_dir_name4, '/', '\'),'\TransformParameters.0.txt')];
    end;
    system(command); %execute external command
    phase2 = toc;
else
    sprintf('The image, the ground truth or the mask don''t exist for this target image: %s\n', image_name{1})
end;

tout20 = phase1 + phase2; % time: segmentation using affine registration (affine with scatter search)

%% SAVING IMAGES
% Convert MHD to PNG: from out_dir_name2 to out_dir_name
command = ['elastixFiles\ConvertToPNG.exe ', strcat(strrep(out_dir_name3, '/', '\'), '\result.mhd')];
system(command); %execute external command
atlasImageDeformed = imread(strcat(out_dir_name3, '\result.png'));
if best_ref ~= -1 %best_ref will be -1 in CT and MRI, since the atlas is not used to select the best reference slice
    hippo_filename = sprintf('%s/%s_newmask_DS_%d_%s.png',out_dir_name,image_name, REFERENCES(best_ref),timestamp); %save extracted ROI
else
    hippo_filename = sprintf('%s/%s_newmask_DS_%s.png',out_dir_name,image_name, timestamp); %save extracted ROI
end;
movefile(strcat(out_dir_name3, '\result.png'),hippo_filename);

atlasImageDeformed = double(atlasImageDeformed/255);

newmask = atlasImageDeformed;

% Save the .mat and .png of the result (the atlas deformed to fit the
% target image) to our_dir_name
save(strcat(out_dir_name,'/',image_name,'_DS.mat'),'newmask');
if best_ref ~= -1 %best_ref will be -1 in CT and MRI, since the atlas is not used to select the best reference slice
    hippo_filename = sprintf('%s/%s_orig_im_newmask_DS_%d_%s.png',out_dir_name,image_name, REFERENCES(best_ref),timestamp); %save extracted ROI
else
    hippo_filename = sprintf('%s/%s_orig_im_newmask_DS_%s.png',out_dir_name,image_name, timestamp); %save extracted ROI
end;

orig_im_resized = imresize(orig_im,size(newmask));
imwrite(im2double(orig_im_resized).*newmask, hippo_filename,'png');

