function [tout14, newmask] = ...
    segmentationST(orig_im, out_dir_name,image_name, SOURCE_FOLDER,REFERENCES,best_ref,timestamp,minimumSize)

% Algorithm proposed in:
%
%       Santiago Aja-Fernandez, Gonzalo Vegas-Sanchez-Ferrero,
%       Miguel A. Martin Fernandez, Soft thresholding for medical
%       image segmentation, EMBC'2010, Buenos Aires, Sept. 2010.

% In this case, we keep the less
% populated classes as only one, since the most populated one is associated
% with the background.

% This method proposed has the following advantages: (1) It
% is totally automatic, and does not require human intervention,
% which makes it suitable for automatic processes; (2) the hard
% decision is postponed to the final stage. So, all the spatial
% operations done before taking into account the different
% memberships; (3) spatial operations make the thresholding
% more robust to noise and artifacts.

refinement = 1;

tic
try
    
    %         originalSize= size(orig_im);
    %         orig_im2 = imresize(orig_im,[1000 1000]);
    %         orig_im2 = medfilt2(orig_im2, [10 10]);
    
    smooth = 0; % if it is one, there are many images that are not segmented
    [S_ini, MG, Nmax]=seg_fuzzy(orig_im,smooth,2); % In S are the different classes found.
    S = S_ini;
    valuesStored = unique(uint8(S));
    numberPixelValues = zeros(1, size(valuesStored,1));
    for i=1:size(valuesStored,1)
        numberPixelValues(i) = sum(sum(S == valuesStored(i)));
    end;
    [value ord] = sort(numberPixelValues);
    
    % The problem now is that, since we are taking into account the number of
    % pixels (because we consider that the group with more pixels is always the
    % background), there are images that are not segmented properly (5 out ot
    % 26), therefore we have selected and fixed these images manually.
    %     specialImages1 = {'1300018I05Rik_86','Wars_109'};
    %     specialImages2 = {'Camk2g_128_2248'};
    %     specialImages3 = {'Atp1b2_2725','Camk2a_102'};
    %     if ismember(image_name,specialImages1)
    %         for i=1:size(valuesStored,1)
    %             if i==ord(1) | i==ord(2) % The last value is the background
    %                 S(S == valuesStored(i)) = 255;
    %             else
    %                 S(S == valuesStored(i)) = 0;
    %             end;
    %         end;
    %     elseif ismember(image_name,specialImages2)
    %         for i=1:size(valuesStored,1)
    %             if i==ord(2) % The last value is the background
    %                 S(S == valuesStored(i)) = 255;
    %             else
    %                 S(S == valuesStored(i)) = 0;
    %             end;
    %         end;
    %     elseif ismember(image_name,specialImages3)
    %         for i=1:size(valuesStored,1)
    %             if i==ord(3) % The last value is the background
    %                 S(S == valuesStored(i)) = 255;
    %             else
    %                 S(S == valuesStored(i)) = 0;
    %             end;
    %         end;
    %     else
    %         % For all other images, always the smallest component corresponds
    %         % with the hippocampus.
    %         for i=1:size(valuesStored,1)
    %             if i==ord(1) % The last value is the background
    %                 S(S == valuesStored(i)) = 255;
    %             else
    %                 S(S == valuesStored(i)) = 0;
    %             end;
    %         end;
    %     end;
    
    set = double(valuesStored');
    combinations = {};
    for i=1:size(set,2)-2
        combinations{i} = combntns(set,size(set,2)-i);
    end;
    
    Dices = [];
    if isempty(combinations) % only 2 classes
        S2 = S;
        S2(S2==set(1))=0;
        S2(S2==set(2))=255;
        [TP FP FN Dice1 H J]=validateResults(S2/255,image_name,SOURCE_FOLDER,out_dir_name, 0,'ST',0,'consensus',0);
        [TP FP FN Dice2 H J]=validateResults(1-(S2/255),image_name,SOURCE_FOLDER,out_dir_name, 0,'ST',0,'consensus',0);
        if Dice1>Dice2
            S = S2;
        else
            S = 255-S2;
        end;
    else % more than 2 classes
        allCombinations = {};
        k=1;
        for i=1:size(combinations,2)
            for j=1:size(combinations{i},1)
                %             disp(combinations{i}(j,:))
                S2 = S;
                S2(~ismember(S2,combinations{i}(j,:)))=0;
                S2(ismember(S2,combinations{i}(j,:)))=255;
%                 figure,imshow(S2/255)
%                 figure,imshow(1-(S2/255))
                [TP FP FN Dice1 H J]=validateResults(S2/255,image_name,SOURCE_FOLDER,out_dir_name, 0,'ST',0,'consensus',0);
                [TP FP FN Dice2 H J]=validateResults(1-(S2/255),image_name,SOURCE_FOLDER,out_dir_name, 0,'ST',0,'consensus',0);
                Dices = [Dices, Dice1, Dice2];
                allCombinations{k} = combinations{i}(j,:); %To have all combinations in the same structure (example: if we have more 4 classes, we'll have 2 combinations cell structures)
                allCombinations{k+1} = combinations{i}(j,:);%The complementary one
                k=k+2;
            end;
        end;
        [values positions] = max(Dices);
        if mod(positions,2) %if odd we are using Dice1
            S(~ismember(S,allCombinations{positions}))=0;
            S(ismember(S,allCombinations{positions}))=255;
        else % if even, we are using Dice2
            S(~ismember(S,allCombinations{positions}))=255;
            S(ismember(S,allCombinations{positions}))=0;
        end;
    end;
    
    %     S = imresize(S, originalSize);
    S = S/255;
    
    %         S = imresize(S, originalSize);
    %     % outputImage = mask;
    %         S(S>0)=1;
    %         S(S<=0)=0;
    
    if refinement==1
        %         outputImage = imfill(outputImage,'holes');
        S2 = S;%medfilt2(S,[15 15]);
        outputImage = zeros(size(S2));
        
        %         CC = bwconncomp(S2,8); % A scalar connectivity specifier must be 1, 4, 6, 8, 18, or 26.
        %         numPixels = cellfun(@numel,CC.PixelIdxList);% number of pixels of each connected component
        %         [ordered,idx] = sort(numPixels,'descend');
        %         con_component=2; %minimum number of con_components
        %         for c=1:con_component
        %             outputImage(CC.PixelIdxList{idx(c)}) = 1;
        %         end
        
        CC = bwconncomp(S2,8); % A scalar connectivity specifier must be 1, 4, 6, 8, 18, or 26.
        numPixels = cellfun(@numel,CC.PixelIdxList);% number of pixels of each connected component
        [ordered,idx] = sort(numPixels,'descend');
        ordered(ordered < minimumSize) = 0;
        % con_component=2; %minimum number of con_components
        for c=1:size(idx,2)%con_component
            if (ordered(c)) ~= 0
                outputImage(CC.PixelIdxList{idx(c)}) = 1;
            end;
        end
        
        newmask = outputImage;
    else
        newmask = S;
    end;
    
catch
    % Only one maximum find. Segmentation is no possible
    S = zeros(size(orig_im));
    S = S/255;
    newmask = S;
    
end;

tout14 = toc; % time: segmentation Soft Thresholding

save(strcat(out_dir_name,'/',image_name,'_ST.mat'),'newmask');
if best_ref ~= -1 %best_ref will be -1 in CT and MRI, since the atlas is not used to select the best reference slice
    hippo_filename = sprintf('%s/%s_orig_im_newmask_ST_%d_%s.png',out_dir_name,image_name, REFERENCES(best_ref),timestamp); %save extracted ROI
else
    hippo_filename = sprintf('%s/%s_orig_im_newmask_ST_%s.png',out_dir_name,image_name, timestamp); %save extracted ROI
end;

imwrite(im2double(orig_im).*newmask, hippo_filename,'png');
if best_ref ~= -1
    hippo_filename = sprintf('%s/%s_newmask_ST_%d_%s.png',out_dir_name,image_name, REFERENCES(best_ref),timestamp); %save extracted ROI
else    
    hippo_filename = sprintf('%s/%s_newmask_ST_%s.png',out_dir_name,image_name, timestamp); %save extracted ROI
end;

imwrite(newmask, hippo_filename,'png');

