% clc
% clear all
% close all
% fclose('all')

%% ---------------------------- IMPORTANT ----------------------------
%% INPUT: inputParameters.txt with all the necessary parameters for the
%% framework:
%% - input file with data
%% - type of problem to solve: minimization (0), maximization (1)
%% - level of significance of the statistical tests (alpha)
%% - number of datasets/functions examined. For example, we could compare 5
%% methods over 10 benchmark functions. If we include 1 means that all
%% images/datasets/functions are going to be considered as only one
%% - save intermediate Figures (histograms, plots,...)
%% - file where the result will be written
%% - minimum number of runs per dataset to study the effect per
%% dataset/image/function

%% 1 - Read parameters file
fidParametersFile = fopen('inputParameters.txt', 'r');
parametersFile=textscan(fidParametersFile,'%s','delimiter','\n');
inputFile = char(strtok(parametersFile{1}(1)));
problemType = str2num(cell2mat(strtok(parametersFile{1}(2)))); %0 -> minimization problem, 1 -> maximization problem
significanceLevel = str2num(cell2mat(strtok(parametersFile{1}(3)))); % significance level: usually 0.01 or 0.05
numberOfFunctions = str2num(cell2mat(strtok(parametersFile{1}(4)))); % number of images? If 1 --> all the results are considered as obtained from the same dataset/function/image
saveFigures = str2num(cell2mat(strtok(parametersFile{1}(5)))); % save Figures?
outputFile = char(strtok(parametersFile{1}(6))); % name of the output file
minimumAllowedNumberOfRuns = str2num(cell2mat(strtok(parametersFile{1}(7)))); % if they're less than this amount we don't study the effect per image/function/dataset
fclose(fidParametersFile);
if (problemType ~= 1) && (problemType ~= 0)
    disp(' ')
    disp('ERROR FOUND!!! The type of problem should be 0 (minimization) or 1 (maximization)');
    disp(' ')
    break;
end;
outputFileWithoutExtension = strrep(outputFile,'.txt','');
outputFileName = strrep(inputFile,'.txt','');
s2 = regexp(outputFileWithoutExtension, '/', 'split');
s3 = regexp(outputFileName, '/', 'split');
outputPath = s2(1);
for i=2:size(s2,2)-1
    outputPath = strcat(outputPath,'/',s2(i));
end;
outputPath = char(strcat(outputPath,'/'));
outputFileName = char(s3(size(s3,2)));
GranadaCodesON = 0;

%% 2 - read csv file with data
try
    M = csvread(inputFile,1,0);
    %% VERY IMPORTANT: CSVREAD fills empty delimited fields with zero. The file SHOULD NOT contain missing values.
catch emptyCSV
    disp(' ')
    disp('ERROR FOUND!!! Error using csvread --> Maybe the name file to open is not correct');
    disp(' ')
    break;
end;
fidInputFile = fopen(inputFile, 'r');
M_2 = textscan(fidInputFile,'%s','delimiter','\n');
fclose(fidInputFile);

%% 3 -  how many columns (methods) and rows (runs per method/function) we have?
numberOfMethodsToCompare = size(M,2);
numberOfRunsPerFunction = size(M,1)/numberOfFunctions;
if numberOfRunsPerFunction ~= round(numberOfRunsPerFunction)
    disp(' ')
    disp('ERROR FOUND!!! Number of runs per function is not an integer');
    disp('Remember: this code considers the same number of runs per function');
    disp('If there are 100 rows in your matrix result and you have 3 benchmark functions in the input file, you will have 33.33 runs per function');
    disp(' ')
    break;
end;

remainString = M_2{1}(1);
i=1;
while ~strcmp(remainString,'')
    [methodsUsed(i), remainString] = strtok(remainString,',');
    i=i+1;
end;


%% 4 - create output file
fileID = fopen(outputFile,'w');


%% 5 - is the data normally distributed?
[KolmogorovSmirnoff_pValues normallyDistributed] = normalityCheck(M,significanceLevel,...
    numberOfMethodsToCompare,methodsUsed,outputPath,outputFileName, saveFigures, fileID);


%% 6 - homoscedasticity?
[Bartlett_pValue homoscedasticity] = homoscedasticityCheck(M,significanceLevel, fileID);


%% 7 - create csv files (to be used by the University of Granada codes, only in the non-parametric case) and
%% partial txt and csv files (including the results obtained by function/dataset/image)
allDatasetsCSVFile = strrep(inputFile,'.txt','.csv');
fileID2 = fopen(allDatasetsCSVFile,'w');
label2 = strcat('Dataset,',char(M_2{1}(1)));
%     label2(size(label2,2))='';
fprintf(fileID2,'%s\n',label2);
fclose(fileID2);
label3 = [];
for i=1:numberOfFunctions
    for j=1:numberOfRunsPerFunction
        label3 = [label3; i];
    end;
end;
matrixToPrintFinal = [label3, M];
dlmwrite(allDatasetsCSVFile, matrixToPrintFinal, '-append', 'delimiter', ',');
boxplot(M,regexp(char(M_2{1}(1)), ',', 'split'))
title('Boxplot containing all results for all methods and functions')
saveas(gcf,strcat(outputPath,'BoxPlot_DatasetsAll_',outputFileName,'.jpg'))
close all;

% Creation of the files with the values obtained per function
if numberOfFunctions > 1 & numberOfRunsPerFunction >= minimumAllowedNumberOfRuns
    for i=1:numberOfFunctions
        fileID3 = fopen(strcat(strrep(allDatasetsCSVFile,'.csv',''),'_Dataset_',num2str(i),'.csv'),'w');
        fileID2 = fopen(strcat(strrep(allDatasetsCSVFile,'.csv',''),'_Dataset_',num2str(i),'.txt'),'w');
        label2 = strcat('Dataset,',char(M_2{1}(1)));
        fprintf(fileID3,'%s\n',label2);
        fprintf(fileID2,'%s\n',char(M_2{1}(1)));
        fclose(fileID3);
        fclose(fileID2);
        label3 = [];
        for j=1:numberOfRunsPerFunction
            label3 = [label3; i];
        end;
        matrixToPrintFinal = [label3, M((numberOfRunsPerFunction*(i-1)+1):(i*numberOfRunsPerFunction),:)];
        dlmwrite(strcat(strrep(allDatasetsCSVFile,'.csv',''),'_Dataset_',num2str(i),'.csv'), matrixToPrintFinal, '-append', 'delimiter', ',');
        dlmwrite(strcat(strrep(allDatasetsCSVFile,'.csv',''),'_Dataset_',num2str(i),'.txt'), matrixToPrintFinal(:,2:size(matrixToPrintFinal,2)), '-append', 'delimiter', ',');
        boxplot(M((numberOfRunsPerFunction*(i-1)+1):(i*numberOfRunsPerFunction),:),regexp(char(M_2{1}(1)), ',', 'split'))
        label = sprintf('Boxplot containing the results for all methods and Function/Dataset %s',num2str(i));
        title(label)
        saveas(gcf,strcat(outputPath,'BoxPlot_Datasets_',num2str(i),'_',outputFileName,'.jpg'))
        close all;
    end;
end;


if numberOfMethodsToCompare > 1
    %% 8 - type of tests to use
    disp(' ');
    if homoscedasticity & normallyDistributed
        non_parametric_tests = 0; % We will use parametric tests
        label = sprintf('PARAMETRIC METHODS WILL BE USED');
        disp(label);
    else
        non_parametric_tests = 1; % We will use non-parametric tests
        label = sprintf('NON-PARAMETRIC METHODS WILL BE USED');
        disp(label);
    end;
    disp(' ');
    fprintf(fileID,strcat(label,'\n'));
    fprintf(fileID,'\n');
    
    
    %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%                          STATISTICAL TESTS
    %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    fprintf(fileID,'------------------------------------------------------\n');
    %%  --------------- NON-PARAMETRIC TESTS ---------------
    if non_parametric_tests
        if numberOfMethodsToCompare == 2
            label = sprintf('TWO METHODS TO COMPARE: Wilcoxon signed-rank test\n');
            disp(label); fprintf(fileID,label);
            label = sprintf('------------------------------------------------------\n');
            disp(label); fprintf(fileID,label);
            if numberOfRunsPerFunction < minimumAllowedNumberOfRuns
                numberOfDatasets = 1;
            else
                numberOfDatasets = numberOfFunctions;
            end;
            [Wilcoxon_pValue] = nonParametricTwoMethods(M,significanceLevel,problemType,numberOfDatasets,...
                numberOfRunsPerFunction,methodsUsed, outputPath,outputFileName, numberOfMethodsToCompare, ...
                saveFigures, fileID,minimumAllowedNumberOfRuns);
        elseif numberOfMethodsToCompare > 2
            label = sprintf('MORE THAN TWO METHODS TO COMPARE: Friedman test\n');
            disp(label); fprintf(fileID,label);
            label = sprintf('------------------------------------------------------\n');
            disp(label); fprintf(fileID,label);
            label = sprintf('-- Global Results: over all functions (considering all functions as only one) --\n');
            disp(label); fprintf(fileID,label);
            [Friedman_pValue,c] = nonParametricMoreThanTwoMethods(M,significanceLevel,problemType,1,methodsUsed,...
                outputPath,outputFileName, numberOfMethodsToCompare,saveFigures, fileID,'All',GranadaCodesON);
            fprintf(fileID,'\n');
            % Now, we can analyze the effect of the function
            if numberOfFunctions > 1 & numberOfRunsPerFunction >= minimumAllowedNumberOfRuns
                label = sprintf('-- Results per function --\n');
                disp(label);
                fprintf(fileID,label);
                disp(' ')
                disp('Now, we study which method is better on every Function/Dataset: ')
                for i=1:numberOfFunctions
                    label = sprintf('- FUNCTION %d -',i);
                    disp(label);
                    fprintf(fileID, strcat(label,'\n'));
                    % for each function we have to check if we can apply a
                    % parametric or non parametric method
                    temporalDataMatrix = [M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,1:numberOfMethodsToCompare)];
                    [Friedman_pValue,c] = nonParametricMoreThanTwoMethods(temporalDataMatrix,significanceLevel,problemType,1,methodsUsed,...
                        outputPath,outputFileName,numberOfMethodsToCompare, saveFigures, fileID,strcat('Dataset_',num2str(i)),GranadaCodesON);
                    disp(' ')
                end;
            end;
        else
            label = sprintf('Only one method has been used: no comparison has been performed ');
            disp(label);
            fprintf(fileID, strcat(label,'\n'));
        end;
        %%  --------------- PARAMETRIC TESTS ---------------
    else
        if numberOfMethodsToCompare == 2
            label = sprintf('TWO METHODS TO COMPARE: t-test\n');
            disp(label); fprintf(fileID,label);
            label = sprintf('------------------------------------------------------\n');
            disp(label); fprintf(fileID,label);
            if numberOfRunsPerFunction < minimumAllowedNumberOfRuns
                numberOfDatasets = 1;
            else
                numberOfDatasets = numberOfFunctions;
            end;
            [Ttest_pValue] = parametricTwoMethods(M,significanceLevel,problemType,numberOfDatasets,...
                numberOfRunsPerFunction,methodsUsed,outputPath,outputFileName,numberOfMethodsToCompare, ...
                saveFigures, fileID,minimumAllowedNumberOfRuns);
        elseif numberOfMethodsToCompare > 2
            label = sprintf('MORE THAN TWO METHODS TO COMPARE: ANOVA test\n');
            disp(label); fprintf(fileID,label);
            label = sprintf('------------------------------------------------------\n');
            disp(label); fprintf(fileID,label);
            label = sprintf('-- Global Results: over all functions (considering all functions as only one) --\n');
            disp(label); fprintf(fileID,label);
            [ANOVA_pValue,c] = parametricMoreThanTwoMethods(M,significanceLevel,problemType,methodsUsed,...
                outputPath,outputFileName,numberOfMethodsToCompare, saveFigures, fileID);
            fprintf(fileID,'\n');
            % Now, we can analyze the effect of the function
            if numberOfFunctions > 1 & numberOfRunsPerFunction >= minimumAllowedNumberOfRuns
                disp(' ')
                label = sprintf('-- Results per function --\n');
                disp(label);
                fprintf(fileID,label);
                disp(' ')
                disp('Now, we study which method is better on every Function/Dataset: ')
                for i=1:numberOfFunctions
                    label = sprintf('- FUNCTION %d -',i);
                    disp(label);
                    fprintf(fileID, strcat(label,'\n'));
                    % for each function we have to check if we can apply a
                    % parametric or non parametric method
                    temporalDataMatrix = [M((1+(i-1)*numberOfRunsPerFunction):numberOfRunsPerFunction*i,1:numberOfMethodsToCompare)];
                    [KolmogorovSmirnoff_pValues normallyDistributed] = normalityCheck(temporalDataMatrix,significanceLevel,...
                        numberOfMethodsToCompare,methodsUsed,outputPath,strcat('Function',num2str(i),outputFileName), saveFigures, fileID);
                    [Bartlett_pValue homoscedasticity] = homoscedasticityCheck(temporalDataMatrix,significanceLevel, fileID);
                    if homoscedasticity & normallyDistributed
                        [ANOVA_pValue,c] = parametricMoreThanTwoMethods(temporalDataMatrix,significanceLevel,problemType,methodsUsed,...
                            outputPath,outputFileName, numberOfMethodsToCompare, saveFigures, fileID);
                    else
                        [Friedman_pValue,c] = nonParametricMoreThanTwoMethods(temporalDataMatrix,significanceLevel,problemType,1,methodsUsed,...
                            outputPath,outputFileName, numberOfMethodsToCompare, saveFigures, fileID,strcat('Dataset_',num2str(i)),GranadaCodesON);
                    end;
                    disp(' ')
                end;
            end;
        else
            label = sprintf('Only one method has been used: no comparison has been performed ');
            disp(label);
            fprintf(fileID, strcat(label,'\n'));
        end;
    end
    
end

fprintf(fileID,'------------------------------------------------------\n');

disp(' ')
label = sprintf('Global Statistical Measurements');
disp(label)
fprintf(fileID,strcat(label,'\n'));
for i=1:size(methodsUsed,2)
    output = sprintf('%s: Mean: %f | Median: %f | Std: %f' ,char(methodsUsed(i)),mean(M(:,i)),median(M(:,i)),std(M(:,i)));
    disp(output)
    fprintf(fileID,strcat(output,'\n'));
end;

fclose('all');


