function [KolmogorovSmirnoff_pValues normallyDistributed] ...
    = normalityCheck(M,significanceLevel,numberOfMethodsToCompare,methodUsed,outputPath,inputFile, saveFigures, fileID)

fprintf(fileID,'NORMALITY CHECK: Kolmogorov-Smirnoff test\n');
for i=1:numberOfMethodsToCompare
    % We remove NaN
    NaN_positions = isnan(M(:,i));
    if sum(NaN_positions) > 0
        M_new = M(~NaN_positions,i);
    else
        M_new = M(:,i);
    end;
    
    % We normalize the data because the kstest compares your data with a
    % N(0,1), but if your data follows a N(2,1) the test will say: Data
    % does not come from a normal distribution.
    if std(M_new) == 0
        M_normale = M_new;
    else
        M_normale = (M_new - mean(M_new))/std(M_new);
        % M_normale = (M(:,i) - mean(M(:,i)))/std(M(:,i));
    end;
        
    % h = kstest(x) performs a Kolmogorov-Smirnov test to compare the
    % values in the data vector x to a standard normal distribution.
    % The null hypothesis is that x has a standard normal distribution.
    % The alternative hypothesis is that x does not have that distribution.
    % The result h is 1 if the test rejects the null hypothesis at the 5% significance level, 0 otherwise.
    [KolmogorovSmirnoff_h(i),KolmogorovSmirnoff_pValues(i),ksstat,cv] = kstest(M_normale,[],significanceLevel);
    if KolmogorovSmirnoff_h(i)==0
        label = sprintf('The results obtained by %s come from a normal distribution',char(methodUsed(i)));
        disp(label);
    else
        label = sprintf('The results obtained by %s do NOT come from a normal distribution',char(methodUsed(i)));
        disp(label);
    end;
    
    fprintf(fileID,'p-value: %d\n',KolmogorovSmirnoff_pValues(i));
    fprintf(fileID,strcat(label,'\n'));
    
    if saveFigures
        % The histogram shows the distribution of data values.
        hist(M(:,i));
        title(sprintf('Histogram of the results obtained by %s',char(methodUsed(i))))
        saveas(gcf,strcat(outputPath,'Histogram_',char(methodUsed(i)),'_',inputFile,'.jpg'))
        close all
        
        % qqplot(X) displays a quantile-quantile plot of the sample quantiles
        % of X versus theoretical quantiles from a normal distribution.
        % If the distribution of X is normal, the plot will be close to linear.
        qqplot(M(:,i));
        title(sprintf('Quantile-quantile plot of the results obtained by %s',char(methodUsed(i))))
        saveas(gcf,strcat(outputPath,'QQplot_',char(methodUsed(i)),'_',inputFile,'.jpg'))
        close all
    end;
end;

if sum(KolmogorovSmirnoff_h) ~= 0
    normallyDistributed = 0;
else
    normallyDistributed = 1;
end;

fprintf(fileID,'\n');
